"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SmartInput = void 0;
var react_1 = __importDefault(require("react"));
var slate_1 = require("slate");
var slate_history_1 = require("slate-history");
var slate_react_1 = require("slate-react");
var sessionSid = '' + Date.now();
function SmartInput(props) {
    var editor = react_1.default.useMemo(function () { return (0, slate_history_1.withHistory)((0, slate_react_1.withReact)((0, slate_1.createEditor)())); }, []);
    var _a = react_1.default.useState(''), completion = _a[0], setCompletion = _a[1];
    var initialValue = [
        {
            type: 'text',
            children: [{ text: props.value, type: undefined }],
        },
    ];
    var renderElement = react_1.default.useCallback(function (p) {
        switch (p.element.type) {
            case 'text':
                return props.renderText(__assign({}, p));
            case 'completion':
                return props.renderSuggestion(__assign(__assign({}, p), { contentEditable: 'false', suppressContentEditableWarning: true }));
            default:
                return react_1.default.createElement(slate_react_1.DefaultElement, __assign({}, p));
        }
    }, [props.renderText, props.renderSuggestion]);
    var acceptSuggestion = function () {
        acceptCompletion(editor, completion);
        setCompletion('');
    };
    var addSuggestion = function (completion) {
        addCompletion(editor, completion);
        setCompletion(completion);
    };
    var clearSuggestion = function () {
        removeCompletion(editor);
        setCompletion('');
    };
    var sendEvents = react_1.default.useMemo(function () {
        return debounce(function (text, context) {
            fetch(props.apiEndpoint || 'https://api.lydian.company/app/events', {
                method: 'POST',
                mode: 'cors',
                headers: {
                    'Content-Type': 'application/json',
                    Authorization: "Bearer ".concat(props.modelId),
                },
                body: JSON.stringify({
                    model: props.modelId,
                    session: sessionSid,
                    input: text,
                    context: context,
                }),
            })
                .then(function (res) {
                if (res.status !== 200) {
                    console.error("Couldn't fetch suggestions");
                    return;
                }
                return res.json();
            })
                .then(function (data) {
                var suggestion = data.suggestion;
                if (suggestion) {
                    addSuggestion(suggestion);
                }
            })
                .catch(function (e) { return console.error(e); });
        }, 200);
    }, [props.modelId]);
    return (react_1.default.createElement(slate_react_1.Slate, { editor: editor, value: initialValue, onChange: function (value) {
            var text = serialize(value.filter(function (n) { return n.type !== 'completion'; }));
            sendEvents(text, {});
            props.onChange(text);
        } },
        react_1.default.createElement(slate_react_1.Editable, { placeholder: props.placeholder, className: props.className, style: props.style, onPaste: function (event) {
                clearSuggestion();
                event.preventDefault();
                var value = event.clipboardData.getData('Text');
                if (!props.multiline) {
                    editor.insertText(value.replaceAll('\n', ''));
                }
                else {
                    editor.insertText(value);
                }
            }, renderElement: renderElement, onBlur: props.onBlur, onFocus: props.onFocus, onKeyDown: function (event) {
                var hasCompletion = completion.trim().length > 0;
                if (event.key === 'Tab' && hasCompletion) {
                    event.preventDefault();
                    acceptSuggestion();
                }
                else if (event.key === 'ArrowRight' && hasCompletion) {
                    event.preventDefault();
                    acceptSuggestion();
                }
                else if (event.key === 'ArrowLeft' && hasCompletion) {
                    clearSuggestion();
                }
                else if ((event.key === 'ArrowDown' || event.ke === 'ArrowUp') && hasCompletion) {
                    clearSuggestion();
                }
                else if (event.key === 'Backspace') {
                    clearSuggestion();
                }
                else if (event.key === 'Enter' && !event.metaKey) {
                    event.preventDefault();
                    if (props.multiline) {
                        clearSuggestion();
                        editor.insertText('\n');
                    }
                }
                else {
                    clearSuggestion();
                }
                if (props.onKeyDown) {
                    props.onKeyDown(event);
                }
            } })));
}
exports.SmartInput = SmartInput;
function isCompletionActive(editor) {
    var completion = editor.children.find(function (n) { return n.type === 'completion'; });
    return !!completion;
}
function isCursorAtEndOfDocument(editor) {
    var sel = editor.selection;
    if (!sel) {
        return false;
    }
    var node = sel.anchor.path[0];
    var offset = sel.anchor.offset;
    if (node !== editor.children.length - 1) {
        return false;
    }
    return offset === slate_1.Node.string(editor.children[node]).length;
}
function acceptCompletion(editor, completion) {
    var isActive = isCompletionActive(editor);
    if (isActive) {
        removeCompletion(editor);
        editor.insertText(completion);
    }
}
function addCompletion(editor, completion) {
    var isActive = isCompletionActive(editor);
    var isCursorEnd = isCursorAtEndOfDocument(editor);
    if (!isActive && isCursorEnd) {
        slate_1.Transforms.insertNodes(editor, {
            type: 'completion',
            children: [{ text: completion }],
        }, {
            at: [editor.children.length],
        });
    }
}
function removeCompletion(editor) {
    var isActive = isCompletionActive(editor);
    if (isActive) {
        slate_1.Transforms.removeNodes(editor, {
            at: [editor.children.length - 1],
        });
    }
}
var serialize = function (nodes) {
    return nodes.map(function (n) { return slate_1.Node.string(n); }).join('\n');
};
var debounce = function (callback, wait) {
    var timeoutId = null;
    return function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        window.clearTimeout(timeoutId);
        timeoutId = window.setTimeout(function () {
            callback.apply(null, args);
        }, wait);
    };
};
//# sourceMappingURL=SmartInput.js.map